vec3

Struct vec3 

#[repr(C)]
pub struct vec3<T>(pub T, pub T, pub T);
3 dimensional vector.

Tuple Fields

0: T1: T2: T

Implementations

Source

impl<T> vec3<T>

pub fn xy(self) -> vec2<T>

Get the first two components as a vec2

pub fn extend(self, w: T) -> vec4<T>

Extend with another component and get a vec4

pub fn map<U, F: Fn(T) -> U>(self, f: F) -> vec3<U>

Map every component (coordinate)

pub fn zip<U>(self, v: vec3<U>) -> vec3<(T, U)>

Zip two vectors together

Source

impl<T: Clone> vec3<T>

pub fn splat(value: T) -> Self

Construct a vector with all components set to specified value

Source

impl<T: UNum> vec3<T>

pub const ZERO: Self

A zero 3-d vector

pub const UNIT_X: Self

A unit X

pub const UNIT_Y: Self

A unit Y

pub const UNIT_Z: Self

A unit Z

Source

impl<T: Copy + Num> vec3<T>

pub fn dot(a: Self, b: Self) -> T

Calculate dot product of two vectors.

Examples
assert_eq!(vec3::dot(vec3(1, 2, 3), vec3(3, 4, 5)), 26);
pub fn cross(a: Self, b: Self) -> Self

Calculate cross product of two vectors.

Examples
assert_eq!(vec3::cross(vec3(1, 2, 3), vec3(3, 4, 5)), vec3(-2, 4, -2));
Source

impl<T: Float> vec3<T>

pub fn normalize(self) -> Self

Normalize a vector.

Examples
let v: vec3<f64> = vec3(1.0, 2.0, 3.0);
assert!((v.normalize().len() - 1.0).abs() < 1e-5);
pub fn normalize_or_zero(self) -> Self

Normalizes a vector unless its length is approximately 0. Can be used to avoid division by 0.

Uses Approx::approx_eq to determine equality to zero

Examples
let v = vec3(1.0, 2.0, 3.0);
assert_eq!(v.normalize_or_zero(), v.normalize());
let v = vec3(1e-10, 1e-10, 1e-10);
assert_eq!(v.normalize_or_zero(), vec3::ZERO);
pub fn len(self) -> T

Calculate length of a vector.

pub fn len_sqr(self) -> T

Calculate squared length of this vector

pub fn into_2d(self) -> vec2<T>

Convert a homogenous 3d vector into 2d

Same as self.xy() / self.z

pub fn clamp_len(self, len_range: impl FixedRangeBounds<T>) -> Self

Clamp vector’s length. Note that the range must be inclusive.

Examples
let v = vec3(1.0, 2.0, 3.0);
assert_eq!(v.clamp_len(..=1.0), v.normalize());
pub fn clamp_coordinates( self, x_range: impl FixedRangeBounds<T>, y_range: impl FixedRangeBounds<T>, z_range: impl FixedRangeBounds<T>, ) -> Self

Clamp vector in range. Note the range must be inclusive.

Examples
let v = vec3(1.0, 2.0, 3.0);
assert_eq!(v.clamp_coordinates(.., 0.0..=1.0, 5.0..), vec3(1.0, 1.0, 5.0));
pub fn transform(self, transform: mat4<T>) -> Self

Apply transformation matrix